;***************************************************************************
;*
;*     PANIC! for Colour GameBoy - SJ 99
;*
;*
;***************************************************************************
; CGB memorymap
;
;0000->3fff	16K Low Cart Page
;4000->7fff	16K Higher Switchable Cart Page
;8000->9fff	8K Vram + 8K CBG Vram bank (8000=Char GFX, 9800=Map data)
;a000->bfff	8K External Battery Backed Ram
;c000->dfff	8K Work Ram, divided into two 4K banks, higher is switchable to achieve 32K
;e000->fdff	reserved
;fe00->fe9f	OAM Sprite co-ord ram
;ff00->ff7f		Hardware registers
;ff80->fffe		128 byte internal Stack ram
;

		Include	"hardware.inc"

		Section	"GBCPanic",Home[0]

Rst00		jp	Main
		ds	5
Rst08		jp	Main
		ds	5
Rst10		jp	Main
		ds	5
Rst18		jp	Main
		ds	5
Rst20		jp	Main
		ds	5
Rst28		jp	Main
		ds	5
Rst30		jp	Main
		ds	5
Rst38		jp	Main
		ds	5
		jp	$ff80		; VBlank Interrupt points to OAM DMA handler
		ds	5
		jp	LCDCInterrupt
		ds	5
		jp	TimerInterrupt
		ds	5
		jp	SerialInterrupt
		ds	5
		jp	HiLoInterrupt
		ds	5

		ds	$100-$68

JumpStart	nop
		jp	Main

NintendoLogo	db	$CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
		db	$00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
		db	$BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E

			;0123456789ABCDEF
CartName		db	"Color PANIC!   "

		db	$80,0,0,0		; CartType:	Colour Gameboy
		db	0		; Rom/Ram config:	0 = Rom only
		db	0		; RomSize:	0 = 32K
		db	0		; RamSize:	0 = none
		dw	$0000		; Manufacturer
		db	0		; Version
		db	0		; Complement check
		dw	0		; Checksum

;***********************************************************************

Main		di
		ld	hl,$fffe
		ld	sp,hl

		call	WaitVBL

		ld	a,0
		ldh	[rLCDC],a		; LCD OFF

		call	ClearGBRam	; Cleanup ram contents
		call	CopyDMAProg	; Put OAM DMA handler program into Stack area
		call	SetupLogoGFX

		ld	a,0
		ldh	[rSCY],a		; Clear scroll position registers
		ldh	[rSCX],a
		ld	a,STATF_LYC	; Select LCD Y line comparitor match flag
		ldh	[rSTAT],a		; LCD Status reg
		ld	a,80
		ldh	[rLYC],a		; Load 80 in line match reg
		ld	a,%00011011
		ldh	[rBGP],a		; Background palette reg, load pixel colour 00 with 11 intensity
		ld	a,IEF_VBLANK
		ldh	[rIE],a		; Interrupt enable reg, enable VBLANk int

		ld	a,%10010111
		ldh	[rLCDC],a		; Lcd ON, BG Char at 8000, BG Bank at 9800, BG ON, OBJ on, OBJ8*16

WaitStartOFF	call	ReadPad
		ld	a,[PadData]
		bit	7,a
		jr	nz,WaitStartOFF

WaitStart		call	ReadPad
		ld	a,[PadData]
		bit	7,a
		jr	z,WaitStart

		call	WaitVBL
		call	SetBlackPalette		

		call	SetupGameGFX

		xor	a		; Initialise character and window co-ords
		ld	[CharX],a
		ld	[CharY],a
		ld	[WindowX],a
		ld	[WindowY],a
		
		ld	a,8		; X co-ord for character
		ld	[CharX],a
		ld	a,5*8		; Y co-ord for character
		ld	[CharY],a

		ld	a,72		; X co-ord for character
		ld	[RobX],a
		ld	a,5*8		; Y co-ord for character
		ld	[RobY],a

		ld	hl,176
		ld	a,h
		ld	[RobSpeedPos],a
		ld	a,l
		ld	[RobSpeedPosl],a
		
		ld	hl,-176
		ld	a,h
		ld	[RobSpeedNeg],a
		ld	a,l
		ld	[RobSpeedNegl],a
		
		ei

DeadLoop	jr	DeadLoop

SetupGameGFX	call	CopyGFXVram
		call	CopyTileVram

		ld	a,1
		ldh	[$4f],a
		call	CopyTileCGVram
		xor	a
		ldh	[$4f],a

		call	CopyCollisionMap

		call	WaitVBL
		call	SetPalette
		ret

SetupLogoGFX	ld	a,$80		; BG0:0 Auto inc
		ldh	[$68],a
   		ld	hl,LogoPalette
		ld	d,64		; Colours * 2
LBGCopyPalette	ld	a,[hl+]
		ldh	[$69],a
		dec	d
		jr	nz,LBGCopyPalette

		ld	hl,LogoTiles
		ld	de,$8000
		ld	bc,256*8*2
		call	VMemoryCopy	; Copy BGChars to Character RAM

		ld	a,1
		ldh	[$4f],a
		ld	hl,LogoTileAttr
		ld	de,$9800
		ld	bc,32*18
		call	VMemoryCopy	; Copy Mapdata to BG data mem

		xor	a
		ldh	[$4f],a
		ld	hl,LogoTileMap
		ld	de,$9800
		ld	bc,32*18
		call	VMemoryCopy	; Copy Mapdata to BG data mem
		ret



SetPalette	ld	a,$80		; BG0:0 Auto inc
		ldh	[$68],a
   		ld	hl,BGPalette
		ld	d,64		; Colours * 2
BGCopyPalette	ld	a,[hl+]
		ldh	[$69],a
		dec	d
		jr	nz,BGCopyPalette
		
		ld	a,$80		; CG0:0 Auto inc
		ldh	[$6a],a
   		ld	hl,CGPalette
		ld	d,64		; Colours * 2
CGCopyPalette	ld	a,[hl+]
		ldh	[$6b],a
		dec	d
		jr	nz,CGCopyPalette
		ret

SetBlackPalette	ld	a,$80		; BG0:0 Auto inc
		ldh	[$68],a
   		ld	hl,BGPalette
		ld	d,64		; Colours * 2
CopyPaletteBLK	xor	a
		ldh	[$69],a
		dec	d
		jr	nz,CopyPaletteBLK
		ret

WaitVBL		push	af
WaitVBLLoop	ldh	a,[rLY]		; LCD Y current line
		cp	144
		jr	nz,WaitVBLLoop
		pop	af
		ret

CopyGFXVram	ld	hl,BGTiles
		ld	de,$8000
		ld	bc,8*8*2
		call	VMemoryCopy	; Copy BGChars to Character RAM

		ld	hl,OBJTiles
		call	CopyOBJVram	; Copy Sprites 0 to 5 from different tile format
		call	CopyOBJVram
		call	CopyOBJVram
		call	CopyOBJVram
		call	CopyOBJVram
		call	CopyOBJVram
		call	CopyOBJVram
		call	CopyOBJVram

		ld	hl,FontTiles
		ld	bc,10*8*2
		call	VMemoryCopy	; Copy BGChars to Character RAM
		ret



CopyOBJVram	ld	bc,16
		call	VMemoryCopy	; Copy BGChars to Character RAM
		ld	b,16
VMemCLoop1	inc	de
		dec	b
		jr	nz,VMemCLoop1
		ld	bc,16
		call	VMemoryCopy	; Copy BGChars to Character RAM
		ld	b,32
VMemCLoop2	dec	de
		dec	b
		jr	nz,VMemCLoop2
		ld	bc,16
		call	VMemoryCopy	; Copy BGChars to Character RAM
		ld	b,16
VMemCLoop3	inc	de
		dec	b
		jr	nz,VMemCLoop3
		ld	bc,16
		call	VMemoryCopy	; Copy BGChars to Character RAM
		ret

CopyTileVram	ld	hl,Map1
		ld	de,$9800
		ld	bc,32*28
		call	VMemoryCopy	; Copy Mapdata to BG data mem
		ret

CopyTileCGVram	ld	hl,Map1CG
		ld	de,$9800
		ld	bc,32*28
		call	VMemoryCopy	; Copy Mapdata to BG data mem
		ret

CopyCollisionMap	ld	hl,Map1
		ld	de,$c200
		ld	bc,32*28
		call	MemoryCopy	; Copy Mapdata to collision map data mem
		ret

ClearGBRam	ld	hl,$8000		; Clear Video RAM
		ld	de,$2000
		call	ClearRam

		ld	hl,$c000		; Clear Work RAM
		ld	de,$2000
		call	ClearRam

		ld	hl,$fe00		; Clear Object RAM
		ld	de,$a0
		call	ClearRam
		ret
		
ClearRam		xor	a
		ld	[hl+],a
		dec	de
		ld	a,d
		or	e
		cp	0
		jr	nz,ClearRam
		ret

CopyDMAProg	ld	hl,DMACopy
		ld	de,$ff80
		ld	b,EndDMACopy-DMACopy

CopyDProgLoop	ld	a,[hl+]
		ld	[de],a
		inc	de
		dec	b
		jr	nz,CopyDProgLoop
		ret

DMACopy		push	af		; DMA program for Stack Ram area
		ld	a,$cf		; $cf00 = tempory work ram OAM co-ord list
		ldh	[$46],a		; set OAM DMA running
		ld	a,40
WaitOAMDMA	dec	a
		jr	nz,WaitOAMDMA
		pop	af
		jp	VBlankInterrupt
		nop
		nop
		nop
		nop			; 18 bytes ?
EndDMACopy

		include	"MainSubsO.asm"


;***********************************************************************
; Data Tables


RGB:		macro
		dw	(\1) + (\2)*32 + (\3)*1024
		endm

BGPalette	incbin	"panicBGpal0.bin"
CGPalette	incbin	"panicOBJpal0.bin"
BGTiles		incbin	"panicBG0.bin"
OBJTiles		incbin	"panicOBJ0.bin"

LogoPalette	incbin	"panlogobgpal.bin"
LogoTileMap	incbin	"panlogomap.bin"
LogoTileAttr	incbin	"panlogocgba.bin"
LogoTiles		incbin	"panlogobg.bin"

FontTiles
DB $7C,$7C,$86,$86,$8A,$8A,$92,$92,$A2,$A2,$C2,$C2,$7C,$7C,$00,$00
DB $10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$00,$00
DB $7C,$7C,$82,$82,$02,$02,$7C,$7C,$80,$80,$80,$80,$FE,$FE,$00,$00
DB $7C,$7C,$82,$82,$02,$02,$3C,$3C,$02,$02,$82,$82,$7C,$7C,$00,$00
DB $80,$80,$88,$88,$88,$88,$88,$88,$FE,$FE,$08,$08,$08,$08,$00,$00
DB $FE,$FE,$80,$80,$80,$80,$FC,$FC,$02,$02,$02,$02,$FC,$FC,$00,$00
DB $7C,$7C,$82,$82,$80,$80,$FC,$FC,$82,$82,$82,$82,$7C,$7C,$00,$00
DB $FE,$FE,$02,$02,$04,$04,$08,$08,$10,$10,$20,$20,$40,$40,$00,$00
DB $7C,$7C,$82,$82,$82,$82,$7C,$7C,$82,$82,$82,$82,$7C,$7C,$00,$00
DB $7C,$7C,$82,$82,$82,$82,$7E,$7E,$02,$02,$02,$02,$02,$02,$00,$00
DB $7C,$7C,$82,$82,$82,$82,$FE,$FE,$82,$82,$82,$82,$82,$82,$00,$00
DB $FC,$FC,$82,$82,$82,$82,$FC,$FC,$82,$82,$82,$82,$FC,$FC,$00,$00
DB $7C,$7C,$82,$82,$80,$80,$80,$80,$80,$80,$82,$82,$7C,$7C,$00,$00
DB $FC,$FC,$82,$82,$82,$82,$82,$82,$82,$82,$82,$82,$FC,$FC,$00,$00
DB $FE,$FE,$80,$80,$80,$80,$F8,$F8,$80,$80,$80,$80,$FE,$FE,$00,$00
DB $FE,$FE,$80,$80,$80,$80,$F8,$F8,$80,$80,$80,$80,$80,$80,$00,$00


Map1		incbin	"panicMap0.bin"
Map1CG		incbin	"panicCGBA0.bin"

;		db	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,6,7,0,0,0,0,0,0,0,0,0,0,6,7,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,2,2,2,1,6,7,1,1,1,1,2,2,2,2,2,1,6,7,1,2,2,2,2,2,2,2,2,1,6,7,1
;		db	1,0,0,0,0,6,7,0,0,1,1,0,0,0,0,0,0,6,7,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,0,0,0,0,6,7,0,0,1,1,0,0,0,0,0,0,6,7,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,0,0,0,0,6,7,0,0,1,1,0,0,6,7,0,0,6,7,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,2,2,2,2,1,1,2,2,1,1,1,1,6,7,1,1,1,1,2,2,2,2,2,2,2,2,2,1,6,7,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,0,0,0,0,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,0,0,0,0,0,0,0,0,6,7,1
;		db	1,6,7,0,0,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,6,7,0,0,0,0,0,0,6,7,1
;		db	1,6,7,1,2,2,2,2,2,2,2,2,2,1,1,2,2,1,1,1,1,6,7,1,2,2,2,2,2,1,1,1
;		db	1,6,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,7,0,0,0,0,0,0,0,0,1
;		db	1,6,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,7,0,0,0,0,0,0,0,0,1
;		db	1,6,7,0,0,0,0,0,0,6,7,0,0,0,0,0,0,6,7,0,0,6,7,0,0,0,0,0,0,0,0,1
;		db	1,1,1,2,2,2,2,2,1,6,7,1,1,1,1,1,1,6,7,1,1,1,1,1,2,2,2,2,2,2,2,1
;		db	1,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,6,7,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,6,7,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,0,0,0,0,0,0,0,0,6,7,0,0,1,1,0,0,6,7,0,0,0,0,0,0,0,0,0,0,0,0,1
;		db	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1


MapJumpTab	dw	000,032,064,096,128,160,192,224
		dw	256,288,320,352,384,416,448,480
		dw	512,544,576,608,640,672,704,736
		dw	768,800,832,864,896,928,960,992

WalkFrameSeqR	db	$08,$0a,$0c,$0e,$10,$12,$0c,$0e
ClimbFrameSeqR	db	$14,$16,$18,$1a,$1c,$1e,$18,$1a
WalkFrameSeqL	db	$0a,$08,$0e,$0c,$12,$10,$0e,$0c


		Section	"BlankSpace",Code
		
		rept	$4000		; 32K of nop blanking
		nop
		endr




;************** Ram Data ******************
;
; CharFlag:
;
; bit 0 - move right
; bit 1 - move left
; bit 2 - move up
; bit 3 - move down
; bit 4 - jump
; bit 5 - 
; bit 6 - 
; bit 7 - action took place
;
; CharFrame:
;
; bit 0 - frame0
; bit 1 - frame1 (binary, frames 0-3)
; bit 2 - direction - horizontally right(0)/left(1), vertically up(0)/down(1)
; bit 3 - moving - horizontal(0), vertical(1)
; bit 4 - jumping - othermotion(0),jumping(1)
; bit 5 - 
; bit 6 - 
; bit 7 - movement took place
;
; JumpFlag:
;
; bit 0 - jump direction right(0)/left(1)
; bit 1 - start jump collision left low15
; bit 2 - start jump collision right low15
; bit 3 - start jump collision middle foot15
; bit 4 - start jump collision middle floor16
; bit 5 - 
; bit 6 - 
; bit 7 - jumping up/down(0) - left/right(1)
;
; ColFlag:		Collisions:
;
; bit 0 - right outer low (15,15)
; bit 1 - right inner low (9,15)
; bit 2 - left outer low (0,15)
; bit 3 - left inner low (6,15)
; bit 4 - foot (7,15)
; bit 5 - floor (7,16)
; bit 6 - 
; bit 7 - 
;
; Char Frame list:
;
; $8 - walk f0
; $c - walk f1&3
; $10 - walk f2
;
; $14 - climb f0
; $18 - climb f1&f3
; $1c - climb f2

		SECTION	"StartupVars",BSS

Val		ds	1
PadData		ds	1
PadDataEdge	ds	1
ColX		ds	1
ColY		ds	1
Hole1		ds	1
Hole2		ds	1
ColFlag		ds	1
ColFlagLadder	ds	1
ColFlagOld	ds	1
ColFlagR		ds	1
ColFlagROld	ds	1
WindowX		ds	1
WindowY		ds	1
RobSpeedPos	ds	1
RobSpeedPosl	ds	1
RobSpeedNeg	ds	1
RobSpeedNegl	ds	1
RobX		ds	1
RobXl		ds	1
RobY		ds	1
RobYl		ds	1
RobFlag		ds	1
RobFallenCount	ds	1
RobFallenFlag	ds	1
CharX		ds	1
CharY		ds	1
CharYNJ		ds	1
CharFlag		ds	1
CharFrame	ds	1
CharFrameCount	ds	1
FallCount		ds	1
JumpFlag		ds	1
JumpCount	ds	1
DiffY		ds	1
DiagFlag		ds	1
