; ********************************************************************************************************************************
;      GameBoy PANIC!      8/5/99 SteveJ
;
; ********************************************************************************************************************************
;

; CGB memorymap
;
;0000->3fff	16K Low Cart Page
;4000->7fff	16K Higher Switchable Cart Page
;8000->9fff	8K Vram + 8K CBG Vram bank
;a000->bfff	8K External Battery Backed Ram
;c000->dfff	8K Work Ram, divided into two 4K banks, higher is switchable to achieve 32K
;e000->fdff	reserved
;fe00->fe9f	OAM Sprite co-ord ram
;ff00->ff7f		Hardware registers
;ff80->fffe		128 byte internal Stack ram
;
;
; locations of interest in memory map:-
;
; 8000+		char GFX data
; 9800+		screen tile data
; c000->c1ff	main vars
; c200->c3ff	collision map data (32*24)
; cf00->cfa0	temp OAM data (40*4)
; ff80+		OAM DMA handler (18?)
; ffffe-		stack


SpriteY		equ	$cf00
SpriteX		equ	$cf01
SpriteChar	equ	$cf02
SpriteFlags	equ	$cf03
Spr0		equ	$00
Spr1		equ	$04
Spr2		equ	$08
Spr3		equ	$0c
Spr4		equ	$10
Spr5		equ	$14
Spr6		equ	$18
Spr7		equ	$1c

ladl		equ	6
ladr		equ	7
wall		equ	1
floor0		equ	2
floor1		equ	3
floor2		equ	4
floor3		equ	5

VBlankInterrupt	push	hl
		push	de
		push	bc
		push	af

		call	SetWindowCoords
		call	ReadPad
		call	CheckPadMove
		call	DoCollisionsC
		call	DoCollisionsR
		call	DoJump
		call	MoveChar
		call	MoveRobot
		call	DoGravity
		;call	DoCollects
		call	AnimateChar
		call	MoveWindowX
		call	MoveWindowY
		call	SetCharOBJCoords
		call	SetRobOBJCoords
		call	ShowDiagnostics

		pop	af
		pop	bc
		pop	de
		pop	hl
		reti

LCDCInterrupt	reti
TimerInterrupt	reti
SerialInterrupt	reti
HiLoInterrupt	reti

TurnLCDOff	ld	a,[_HW+rIE]
		push	af
		and	a,~IEF_VBLANK
		ld	[_HW+rIE],a
		ld	a,[_HW+rLCDC]
		and	a,(~LCDCF_ON)&$FF
		ld	[_HW+rLCDC],a
		pop	af
		ld	[_HW+rIE],a
		ret

MemoryCopy	inc	b
		inc	c
		jr	SkipMemCopy

MemCopyLoop	ld	a,[hl+]
		ld	[de],a
		inc	de
SkipMemCopy	dec	c
		jr	nz,MemCopyLoop
		dec	b
		jr	nz,MemCopyLoop
		ret

VMemoryCopy	inc	b
		inc	c
		jr	SkipVMemCopy

VMemCopyLoop	ldh	a,[rSTAT]          	; Wait until you can write to vram
		and	2              
		jr	nz,VMemCopyLoop

		ld	a,[hl+]
		ld	[de],a
		inc	de
SkipVMemCopy	dec	c
		jr	nz,VMemCopyLoop
		dec	b
		jr	nz,VMemCopyLoop
		ret

;************************************************************************

SetCharOBJCoords	ld	a,[WindowX]
		ld	b,a
		ld	a,[CharX]
		sub	a,b
		add	8
		ld	[Spr4+SpriteX],a
		add	8
		ld	[Spr5+SpriteX],a

		ld	a,[WindowY]
		ld	b,a
		ld	a,[CharY]
		sub	a,b
		add	16
		ld	[Spr4+SpriteY],a
		ld	[Spr5+SpriteY],a
		ret

;************************************************************************

SetRobOBJCoords	ld	a,[WindowX]
		ld	b,a
		ld	a,[RobX]
		sub	a,b
		add	8
		ld	[Spr2+SpriteX],a
		add	8
		ld	[Spr3+SpriteX],a

		ld	a,[WindowY]
		ld	b,a
		ld	a,[RobY]
		sub	a,b
		add	16
		ld	[Spr2+SpriteY],a
		ld	[Spr3+SpriteY],a
		
		ld	a,1
		ld	[Spr2+SpriteFlags],a
		ld	a,1
		ld	[Spr3+SpriteFlags],a
		
		ld	a,[RobFlag]
		bit	7,a
		jr	nz,RobFallFrame

RobNormFrame	ld	a,32
		ld	[Spr2+SpriteChar],a
		ld	a,34
		ld	[Spr3+SpriteChar],a		
		ret

RobFallFrame	ld	a,36
		ld	[Spr2+SpriteChar],a
		ld	a,38
		ld	[Spr3+SpriteChar],a		
		ret

;************************************************************************

SetWindowCoords	ld	a,[WindowX]
		ld	[$ff43],a
		ld	a,[WindowY]
		ld	[$ff42],a
		ret

MoveWindowX	ld	a,[CharX]
		ld	b,$28
		sub	a,b
		jp	c,CharXBorderL	; jump if character X pos is less than left border of $20
		ld	a,[CharX]
		ld	b,$c8
		sub	a,b
		jp	nc,CharXBorderR	; jump if greater than right border of $e0

		ld	a,[CharX]
		ld	b,a
		ld	a,[WindowX]
		add	a,$68
		sub	a,b
		jp	c,CharXWindR	; jump if (win+7) is less than charX

		ld	a,[CharX]
		ld	b,a
		ld	a,[WindowX]
		add	a,$28
		sub	a,b
		jp	nc,CharXWindL	; jump if (win+3) is greater than charX
		ret

CharXBorderL	xor	a
		ld	[WindowX],a
		ret
CharXBorderR	ld	a,255-160
		ld	[WindowX],a
		ret
CharXWindR	ld	b,a
		ld	a,[WindowX]
		sub	a,b
		ld	[WindowX],a
		ret
CharXWindL	ld	b,a
		ld	a,[WindowX]
		sub	a,b
		ld	[WindowX],a
		ret

;************************************************************************

MoveWindowY	ld	a,[CharY]
		ld	b,$28
		sub	a,b
		jp	c,CharYBorderU	; jump if character Y pos is less than left border of $20
		ld	a,[CharY]
		ld	b,$b8
		sub	a,b
		jp	nc,CharYBorderD	; jump if greater than right border of $e0

		ld	a,[CharY]
		ld	b,a
		ld	a,[WindowY]
		add	a,$58
		sub	a,b
		jp	c,CharYWindU	; jump if (win+7) is less than charY

		ld	a,[CharY]
		ld	b,a
		ld	a,[WindowY]
		add	a,$28
		sub	a,b
		jp	nc,CharYWindD	; jump if (win+3) is greater than charY
		ret

CharYBorderU	xor	a
		ld	[WindowY],a
		ret
CharYBorderD	ld	a,255-144
		ld	[WindowY],a
		ret
CharYWindU	ld	b,a
		ld	a,[WindowY]
		sub	a,b
		ld	[WindowY],a
		ret
CharYWindD	ld	b,a
		ld	a,[WindowY]
		sub	a,b
		ld	[WindowY],a
		ret

;************************************************************************

CheckPadMove	ld	a,[PadData]
		cp	%11110000
		jr	z,ResetDetected
		bit	6,a
		jr	nz,DiagnosON
		bit	7,a
		jr	nz,DiagnosOFF

		ld	a,[PadData]
		and	%00111111
		jr	nz,PadMoved
		ld	[CharFlag],a
		ret
PadMoved	or	128
		ld	[CharFlag],a
		ret

ResetDetected	rst	00

DiagnosON	ld	a,1
		ld	[DiagFlag],a
		ret
DiagnosOFF	xor	a
		ld	[DiagFlag],a
		ret

		
;************************************************************************

AnimateChar	ld	a,[CharFrame]
		bit	7,a
		jp	z,StandAnimFrame
		and	%01111111
		ld	[CharFrame],a
		ld	a,[CharFrameCount]
		inc	a
		cp	3
		jp	z,NewAnimFrame
		ld	[CharFrameCount],a
NoAnimFrames	ret

StandAnimFrame	ld	a,[CharFrame]
		bit	3,a
		jp	nz,SetCharFrame

StandFrame1	and	%01111100
		or	%00000001
		jp	SetCharFrame

NewAnimFrame	xor	a
		ld	[CharFrameCount],a
		ld	a,[CharFrame]
		inc	a
		and	3
		ld	b,a
		ld	a,[CharFrame]
		and	%11111100
		or	b
SetCharFrame	ld	[CharFrame],a
		bit	2,a
		jp	z,AnimFrameRight
AnimFrameLeft	bit	3,a
		jp	z,AnimFrameWalkL
AnimFrameClimbL	call	AnimUNFlipSprites
		ld	hl,ClimbFrameSeqR
		jp	DoFrameNo
AnimFrameWalkL	call	AnimFlipSprites
		ld	hl,WalkFrameSeqL
		jp	DoFrameNo

AnimFrameRight	bit	3,a
		jp	z,AnimFrameWalkR
AnimFrameClimbR	call	AnimUNFlipSprites
		ld	hl,ClimbFrameSeqR
		jp	DoFrameNo
AnimFrameWalkR	call	AnimUNFlipSprites
		ld	hl,WalkFrameSeqR
		jp	DoFrameNo

AnimFlipSprites	ld	hl,Spr4+SpriteFlags
		set	5,[hl]
		ld	hl,Spr5+SpriteFlags
		set	5,[hl]
		ret
AnimUNFlipSprites	ld	hl,Spr4+SpriteFlags
		res	5,[hl]
		ld	hl,Spr5+SpriteFlags
		res	5,[hl]
		ret

DoFrameNo	ld	a,[CharFrame]
		rl	a
		and	%00000110
		ld	b,0
		ld	c,a
		add	hl,bc
		ld	a,[hl+]
		ld	[Spr4+SpriteChar],a
		ld	a,[hl+]
		ld	[Spr5+SpriteChar],a
		ret
		
;************************************************************************
				
DoGravity		ld	a,[JumpCount]
		cp	0
		jp	nz,NoFall

		ld	b,3
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,FallCancel
		cp	floor0
		jp	z,FallCancel
		cp	floor1
		jp	z,FallCancel
		cp	floor2
		jp	z,FallCancel
		cp	ladl
		jp	z,FallCancel
		cp	ladr
		jp	z,FallCancel

		ld	b,7
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,FallCancel
		cp	floor0
		jp	z,FallCancel
		cp	floor1
		jp	z,FallCancel
		cp	floor2
		jp	z,FallCancel
		cp	ladl
		jp	z,FallCancel
		cp	ladr
		jp	z,FallCancel

		ld	b,12
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,FallCancel
		cp	floor0
		jp	z,FallCancel
		cp	floor1
		jp	z,FallCancel
		cp	floor2
		jp	z,FallCancel
		cp	ladl
		jp	z,FallCancel
		cp	ladr
		jp	z,FallCancel

NoFallCancel	
		
Fall		ld	a,[FallCount]
		inc	a
		ld	[FallCount],a
		;cp	12
		;jp	nc,SingleFallStep
	
SingleFallStep	;ld	a,[FallCount]
		;and	3
		;cp	3
		;jr	z,FallMoveLR
		ld	a,[CharY]
		inc	a
		ld	[CharY],a
		ret

FallMoveLR	ld	a,[CharY]
		inc	a
		ld	[CharY],a
		ld	a,[CharFrame]
		bit	2,a
		jp	z,DoCharMoveR
		jp	DoCharMoveL
		
FallCancel	ld	b,3
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,NoFallCancel
		cp	floor0
		jp	z,NoFallCancel
		cp	floor1
		jp	z,NoFallCancel
		cp	floor2
		jp	z,NoFallCancel

		ld	b,7
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,NoFallCancel
		cp	floor0
		jp	z,NoFallCancel
		cp	floor1
		jp	z,NoFallCancel
		cp	floor2
		jp	z,NoFallCancel

		ld	b,12
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,NoFallCancel
		cp	floor0
		jp	z,NoFallCancel
		cp	floor1
		jp	z,NoFallCancel
		cp	floor2
		jp	z,NoFallCancel


		xor	a
		ld	[FallCount],a
NoFall		ret
		
;************************************************************************

DoCollects	ld	b,3
		ld	c,14
		call	MapCharCollision
		cp	4
		jp	z,CollectEgg
		cp	5
		jp	z,CollectCorn
		ld	b,12
		ld	c,14
		call	MapCharCollision
		cp	4
		jp	z,CollectEgg
		cp	5
		jp	z,CollectCorn
		ret
		
CollectEgg	jp	CollectClearMap
		ret
CollectCorn	jp	CollectClearMap
		ret

CollectClearMap	ld	hl,$9800
		add	hl,bc
		add	hl,de
		xor	a
		ld	[hl],a
		ret

;************************************************************************

DoJump		ld	a,[JumpCount]
		cp	0
		jp	nz,DoingJump	; Check if jump is actually counting
		ret
DoingJump	cp	37
		jp	z,EndOfJump	; Check for the end of the jump sequence

DoVertJump	ld	a,[JumpCount]
		ld	hl,JumpTab
		ld	c,a
		ld	b,0
		inc	a
		ld	[JumpCount],a
		add	hl,bc
		ld	a,[hl]
		cp	1
		jp	z,JumpCharU
		cp	2
		jp	z,JumpCharD
		ret
		
EndOfJump	ld	a,[JumpFlag]
		ld	b,a
		xor	a
		ld	[JumpFlag],a
		ld	[JumpCount],a
		ld	hl,CharFrame
		res	4,[hl]
		res	7,[hl]

		bit	2,b
		ret	z

		xor	a
		ld	[JumpFlag],a

		ld	b,4
		ld	c,16
		call	MapCharCollision
		ld	[Hole1],a
		ld	b,12
		ld	c,16
		call	MapCharCollision
		ld	[Hole2],a

		ld	a,[Hole1]
		cp	floor0
		jr	z,Check2Hole2
		cp	floor1
		jr	z,Check3Hole2
		cp	floor2
		jr	z,Check4Hole2
		ret

Check2Hole2	ld	a,[Hole2]
		cp	floor0
		ret	nz
		ld	a,floor1
		jr	SetHole
Check3Hole2	ld	a,[Hole2]
		cp	floor1
		ret	nz
		ld	a,floor2
		jr	SetHole
Check4Hole2	ld	a,[Hole2]
		cp	floor2
		ret	nz
		ld	a,floor3
		jr	SetHole

SetHole		ld	b,4
		ld	c,16
		call	PlaceBlockScreen
		ld	b,12
		ld	c,16
		call	PlaceBlockScreen
		ret

JumpTab		db	1,1,1,1,1,1,0,1,1,1,0,1,0,1,0,1,0,1,0
		db	0,0,2,0,2,0,2,0,2,2,2,0,2,2,2,2,2,2

JumpCharU	ld	a,[JumpCount]
		sub	3
		jp	c,NotLeftFloorYet

		ld	a,[CharFlag]
		bit	2,a
		jr	nz,JumpLadCheckU
		bit	3,a
		jr	nz,JumpLadCheckU
		
		ld	a,[JumpFlag]		; Check if jumped off ladder
		bit	1,a
		jr	nz,JumpLadCheckU
		jr	NotLeftFloorYet

JumpLadCheckU	ld	a,[ColFlag]
		bit	1,a
		jp	nz,EndOfJump

NotLeftFloorYet	ld	a,[CharY]
		dec	a
		ld	[CharY],a
		ret

JumpCharD	ld	a,[CharFlag]
		bit	2,a
		jr	nz,JumpLadCheckD
		bit	3,a
		jr	nz,JumpLadCheckD
		
		ld	a,[JumpFlag]		; Check if jumped off ladder
		bit	1,a
		jr	nz,JumpLadCheckD
		jr	ContJumpD

JumpLadCheckD	ld	a,[ColFlag]
		bit	1,a
		jp	nz,EndOfJump
		
ContJumpD	ld	a,[ColFlag]
		bit	0,a
		jp	nz,EndOfJump
		ld	a,[CharY]
		inc	a
		ld	[CharY],a
		ret

CheckLadderJump	ld	b,0
		ld	c,3
		call	MapCharCollision
		cp	ladl
		jp	nz,NoJumpLad
		
		ld	b,15
		ld	c,3
		call	MapCharCollision
		cp	ladr
		jp	nz,NoJumpLad

JumpOntoLadder	xor	a
		ld	[JumpCount],a
		ld	[JumpFlag],a
NoJumpLad	ret

;************************************************************************

DoCollisionsC	ld	hl,ColFlag
		ld	a,[hl]
		ld	[ColFlagOld],a
		xor	a
		ld	[hl],a

CheckCol0a	ld	b,3		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,SetCol0
		cp	floor0
		jp	z,SetCol0
		cp	floor1
		jp	z,SetCol0
		cp	floor2
		jp	z,SetCol0

CheckCol0b	ld	b,6
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,SetCol0
		cp	floor0
		jr	z,SetCol0
		cp	floor1
		jr	z,SetCol0
		cp	floor2
		jr	z,SetCol0

CheckCol0c	ld	b,9
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,SetCol0
		cp	floor0
		jr	z,SetCol0
		cp	floor1
		jr	z,SetCol0
		cp	floor2
		jr	z,SetCol0

CheckCol0d	ld	b,12
		ld	c,16
		call	MapCharCollision
		cp	wall
		jp	z,SetCol0
		cp	floor0
		jr	z,SetCol0
		cp	floor1
		jr	z,SetCol0
		cp	floor2
		jr	z,SetCol0
		jr	CheckCol1
SetCol0		
		set	0,[hl]
		
		ld	b,3		; Check 0abcd - Check for platform under player
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,ReSetCol0
		cp	floor0
		jp	z,ReSetCol0
		cp	floor1
		jp	z,ReSetCol0
		cp	floor2
		jp	z,ReSetCol0

CheckCol0br	ld	b,6
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,ReSetCol0
		cp	floor0
		jp	z,ReSetCol0
		cp	floor1
		jp	z,ReSetCol0
		cp	floor2
		jp	z,ReSetCol0
		
CheckCol0cr	ld	b,9
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,ReSetCol0
		cp	floor0
		jp	z,ReSetCol0
		cp	floor1
		jp	z,ReSetCol0
		cp	floor2
		jp	z,ReSetCol0

CheckCol0dr	ld	b,12
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,ReSetCol0
		cp	floor0
		jp	z,ReSetCol0
		cp	floor1
		jp	z,ReSetCol0
		cp	floor2
		jp	z,ReSetCol0
		jr	CheckCol1
ReSetCol0		
		res	0,[hl]
		
CheckCol1	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,7
		call	MapCharCollision
		cp	ladl
		jp	nz,CheckCol2
		ld	b,15
		ld	c,7
		call	MapCharCollision
		cp	ladr
		jp	nz,CheckCol2
		set	1,[hl]

CheckCol2	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,16
		call	MapCharCollision
		cp	ladl
		jp	nz,CheckCol3
		ld	b,15
		ld	c,16
		call	MapCharCollision
		cp	ladr
		jp	nz,CheckCol3
		set	2,[hl]

CheckCol3	ld	b,255		; Check 2 - check for wall hit to left of player, 8
		ld	c,15
		call	MapCharCollision
		cp	wall
		jr	z,SetCol3
		;cp	floor0
		;jr	z,SetCol3
		;cp	floor1
		;jr	z,SetCol3
		;cp	floor2
		;jr	z,SetCol3
		jr	CheckCol4
SetCol3		
		set	3,[hl]

CheckCol4	ld	b,16		; Check 3 - check for wall hit to right of player, 8
		ld	c,15
		call	MapCharCollision
		cp	wall
		jr	z,SetCol4
		;cp	floor0
		;jr	z,SetCol4
		;cp	floor1
		;jr	z,SetCol4
		;cp	floor2
		;jr	z,SetCol4
		jr	CheckCol5a
SetCol4		
		set	4,[hl]

CheckCol5a	ld	b,255		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapCharCollision
		cp	wall
		jr	z,SetCol5a
		cp	floor0
		jr	z,SetCol5a
		cp	floor1
		jr	z,SetCol5a
		cp	floor2
		jr	z,SetCol5a
		cp	floor3
		jr	z,SetCol5a
		jr	CheckCol6a
SetCol5a		ld	b,255
		ld	c,15
		call	MapCharCollision
		cp	wall
		jr	z,CheckCol6a
		cp	floor0
		jr	z,CheckCol6a
		cp	floor1
		jr	z,CheckCol6a
		cp	floor2
		jr	z,CheckCol6a
		cp	floor3
		jr	z,CheckCol6a
		set	5,[hl]

CheckCol6a	ld	b,16		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapCharCollision
		cp	wall
		jr	z,SetCol6a
		cp	floor0
		jr	z,SetCol6a
		cp	floor1
		jr	z,SetCol6a
		cp	floor2
		jr	z,SetCol6a
		cp	floor3
		jr	z,SetCol6a
		jr	CheckCol7
SetCol6a		ld	b,16
		ld	c,15
		call	MapCharCollision
		cp	wall
		jr	z,CheckCol7
		cp	floor0
		jr	z,CheckCol7
		cp	floor1
		jr	z,CheckCol7
		cp	floor2
		jr	z,CheckCol7
		cp	floor3
		jr	z,CheckCol7
		set	6,[hl]

CheckCol7	ld	b,3		; Check 0abcd - Check for ladder under player
		ld	c,16
		call	MapCharCollision
		cp	ladl
		jp	z,SetCol7a
		cp	ladr
		jp	z,SetCol7a
		jp	CheckCol7b
SetCol7a		
		set	7,[hl]

CheckCol7b	ld	b,6
		ld	c,16
		call	MapCharCollision
		cp	ladl
		jp	z,SetCol7b
		cp	ladr
		jp	z,SetCol7b
		jr	CheckCol7c
SetCol7b		
		set	7,[hl]

CheckCol7c	ld	b,9
		ld	c,16
		call	MapCharCollision
		cp	ladl
		jp	z,SetCol7c
		cp	ladr
		jp	z,SetCol7c
		jr	CheckCol7d
SetCol7c		
		set	7,[hl]

CheckCol7d	ld	b,12
		ld	c,16
		call	MapCharCollision
		cp	wall
		cp	ladl
		jp	z,SetCol7d
		cp	ladr
		jp	z,SetCol7d
		jr	EndCheckCol
SetCol7d		
		set	7,[hl]

EndCheckCol	ret
		
; ColFlag:		Collisions:
;
; bit 0 - Platform (3-12,16)
; bit 1 - Ladder (0-15,7)
; bit 2 - Ladder (0-15,15)
; bit 3 - Wall Left (0,15)
; bit 4 - Wall Right (15,15)
; bit 5 - Platform at Left of foot (-1,15-16)
; bit 6 - Platform at Right of foot (16,15-16)
; bit 7 - Ladders underfoot (3-12,16)
;

;************************************************************************

MoveChar	call	SetBackBlack
		ld	a,[JumpCount]
		cp	0
		jp	nz,JumpHorizMove	; No moving char L/R according to Pad when jumping

		ld	a,[CharFlag]
		bit	4,a
		jp	nz,SetCharJump
		bit	5,a
		jp	nz,SetCharHit

		ld	a,[CharY]
		ld	[CharYNJ],a

		ld	a,[CharFlag]
		bit	7,a
		jp	nz,MoveCharAction

NoWalkFallEdge	ret

MoveCharAction	ld	hl,CharFrame
		
CheckVertMove	ld	a,[CharFlag]
		bit	2,a
		jp	nz,MoveCharU
		bit	3,a
		jp	nz,MoveCharD

CheckHorizMove	ld	a,[CharFlag]
		bit	0,a
		jr	nz,MoveCharR
		bit	1,a
		jp	nz,MoveCharL
NoHorizMove	ret

JumpHorizMove	ld	a,[JumpFlag]
		bit	7,a
		jr	z,NoHorizMove
		bit	1,a
		jr	nz,bnm
vbn		ld	a,[JumpFlag]
		bit	2,a
		jr	nz,NoHorizMove
		bit	0,a
		jr	z,MoveCharR
		jp	MoveCharL

bnm		;call	SetBackPurple
		jr	vbn

MoveCharR	ld	a,[ColFlag]
		bit	4,a
		;jp	nz,NoCharMoveR
		jp	nz,BounceWallR

SkipWallCheckR	ld	a,[JumpFlag]
		bit	1,a
		jr	nz,CharOnPlatR
		ld	a,[ColFlag]
		bit	0,a
		jr	nz,CharOnPlatR
		bit	5,a
		jr	nz,CharOnPlatR
		bit	6,a
		jr	nz,CharOnPlatR
		ld	a,[JumpCount]
		cp	0
		jr	nz,CharOnPlatR
		ld	a,[ColFlag]
		cp	128
		jr	z,CharOnPlatR
		jr	NoCharMoveR

CharOnPlatR	ld	a,[ColFlag]
		and	7
		cp	%011
		jr	z,DoCharMoveR

		bit	1,a
		jp	z,DoCharMoveR
		ld	a,[CharFlag]
		bit	2,a
		jr	nz,NoCharMoveR
		bit	3,a
		jr	nz,NoCharMoveR
		
DoCharMoveR	ld	a,[CharX]
		cp	240		; 256-16
		jp	z,NoCharMoveR
		inc	a
		ld	[CharX],a
		ld	hl,CharFrame
		res	2,[hl]
		res	3,[hl]
		set	7,[hl]
		ret

BounceWallR	ld	a,[JumpFlag]
		bit	1,a
		jr	z,NoCharMoveR
		set	0,a
		ld	[JumpFlag],a

NoCharMoveR	
RangeLadChkR	ld	a,[ColFlag]
		bit	2,a
		ret	z

		xor	a
		ld	[ColFlagLadder],a

CheckLadDR	ld	b,16
		ld	c,23
		call	MapCharCollision
		cp	wall
		jp	z,FoundLadDR
		cp	floor0
		jp	z,FoundLadDR
		cp	floor1
		jr	z,FoundLadDR
		cp	floor2
		jp	z,FoundLadDR
		cp	floor3
		jp	z,FoundLadDR
		jp	CheckLadUR
FoundLadDR	ld	hl,ColFlagLadder
		set	0,[hl]

CheckLadUR	ld	b,16
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,FoundLadUR
		cp	floor0
		jp	z,FoundLadUR
		cp	floor1
		jp	z,FoundLadUR
		cp	floor2
		jp	z,FoundLadUR
		cp	floor3
		jp	z,FoundLadUR
		jp	DoneCheckLadR
FoundLadUR	ld	hl,ColFlagLadder
		set	1,[hl]
		
DoneCheckLadR	ld	hl,ColFlagLadder
		ld	a,[hl]
		cp	%01
		jp	z,MoveCharD
		cp	%10
		jp	z,MoveCharU
		ret

		
MoveCharL	ld	a,[ColFlag]
		bit	3,a
		jp	nz,NoCharMoveL
SkipWallCheckL	ld	a,[ColFlag]
		bit	0,a
		jp	nz,CharOnPlatL
		bit	5,a
		jp	nz,CharOnPlatL
		bit	6,a
		jp	nz,CharOnPlatL
		ld	a,[JumpCount]
		cp	0
		jp	nz,CharOnPlatL
		ld	a,[ColFlag]
		cp	128
		jp	z,CharOnPlatL
		jp	NoCharMoveL

CharOnPlatL	ld	a,[ColFlag]
		and	7
		cp	%011
		jr	z,DoCharMoveL
		
		bit	1,a
		jp	z,DoCharMoveL
		ld	a,[CharFlag]
		bit	2,a
		jr	nz,NoCharMoveL
		bit	3,a
		jr	nz,NoCharMoveL
		
DoCharMoveL	ld	a,[CharX]
		cp	8
		jp	z,NoCharMoveL
		dec	a
		ld	[CharX],a
		ld	hl,CharFrame
		set	2,[hl]
		res	3,[hl]
		set	7,[hl]
		ret
NoCharMoveL	
RangeLadChkL	ld	a,[ColFlag]
		bit	2,a
		ret	z

		xor	a
		ld	[ColFlagLadder],a

CheckLadDL	ld	b,255
		ld	c,23
		call	MapCharCollision
		cp	wall
		jp	z,FoundLadDL
		cp	floor0
		jp	z,FoundLadDL
		cp	floor1
		jp	z,FoundLadDL
		cp	floor2
		jp	z,FoundLadDL
		cp	floor3
		jp	z,FoundLadDL
		jp	CheckLadUL
FoundLadDL	ld	hl,ColFlagLadder
		set	0,[hl]

CheckLadUL	ld	b,255
		ld	c,15
		call	MapCharCollision
		cp	wall
		jp	z,FoundLadUL
		cp	floor0
		jp	z,FoundLadUL
		cp	floor1
		jp	z,FoundLadUL
		cp	floor2
		jp	z,FoundLadUL
		cp	floor3
		jp	z,FoundLadUL
		jp	DoneCheckLadL
FoundLadUL	ld	hl,ColFlagLadder
		set	1,[hl]
		
DoneCheckLadL	ld	hl,ColFlagLadder
		ld	a,[hl]
		cp	%01
		jp	z,MoveCharD
		cp	%10
		jp	z,MoveCharU
		ret
	
MoveCharU	ld	a,[CharFrame]
		bit	3,a
		jr	z,DoCharMoveU

CheckPlatUL	ld	a,[ColFlag]
		bit	5,a
		jr	z,CheckPlatUR
		ld	a,[CharFlag]
		bit	1,a
		jp	nz,DoCharMoveL

CheckPlatUR	ld	a,[ColFlag]
		bit	6,a
		jr	z,DoCharMoveU
		ld	a,[CharFlag]
		bit	0,a
		jp	nz,DoCharMoveR

DoCharMoveU	ld	a,[ColFlag]
		bit	1,a
		jr	z,NoCharMoveU
		
		ld	a,[CharY]
		cp	0
		jr	z,NoCharMoveU
		dec	a
		ld	[CharY],a
		ld	hl,CharFrame
		res	2,[hl]
		set	3,[hl]		
		set	7,[hl]		
		ld	a,[CharFlag]
		ret
NoCharMoveU	
RangeLadChkU	;ld	a,[JumpFlag]
		;bit	1,a
		;jp	nz,CheckHorizMove	; Check if jumped off ladder
		
		ld	a,[CharFlag]
		bit	0,a
		jr	nz,LadChkUR		; Skip range left if moving right
		ld	b,0
		ld	c,7
		call	MapCharCollision
		cp	ladl
		jp	z,MoveCharL
		
LadChkUR	ld	a,[CharFlag]
		bit	1,a
		jp	nz,CheckHorizMove	; Skip range right if moving left
		ld	b,15
		ld	c,7
		call	MapCharCollision
		cp	ladr
		jp	z,MoveCharR
		
		jp	CheckHorizMove
	
MoveCharD	ld	a,[CharFrame]
		bit	3,a
		jr	z,DoCharMoveD

CheckPlatDL	ld	a,[ColFlag]
		bit	5,a
		jr	z,CheckPlatDR
		ld	a,[CharFlag]
		bit	1,a
		jp	nz,DoCharMoveL

CheckPlatDR	ld	a,[ColFlag]
		bit	6,a
		jr	z,DoCharMoveD
		ld	a,[CharFlag]
		bit	0,a
		jp	nz,DoCharMoveR

DoCharMoveD	ld	a,[ColFlag]
		bit	2,a
		jr	z,NoCharMoveD
		
		ld	a,[CharY]
		cp	176-8
		jr	z,NoCharMoveD
		inc	a
		ld	[CharY],a
		ld	hl,CharFrame
		set	2,[hl]
		set	3,[hl]
		set	7,[hl]
		ld	a,[CharFlag]
		ret
NoCharMoveD	
RangeLadChkD	;ld	a,[JumpFlag]
		;bit	1,a
		;jp	nz,CheckHorizMove	; Check if jumped off ladder
		
		ld	a,[CharFlag]
		bit	0,a
		jr	nz,LadChkDR		; Skip range left if moving right
		ld	b,0
		ld	c,16
		call	MapCharCollision
		cp	ladl
		jp	z,MoveCharL
		
LadChkDR	ld	a,[CharFlag]
		bit	1,a
		jp	nz,CheckHorizMove	; Skip range right if moving left
		ld	b,15
		ld	c,16
		call	MapCharCollision
		cp	ladr
		jp	z,MoveCharR
		
		jp	CheckHorizMove

;************************************************************************

SetCharHit	ld	a,[JumpCount]
		cp	0
		jp	z,SetNewHit
		jp	CheckVertMove

SetCharJump	ld	a,[JumpCount]
		cp	0
		jp	z,SetNewJump
		jp	CheckVertMove

SetNewHit	ld	a,[ColFlag]
		bit	1,a
		ret	nz
		bit	2,a
		ret	nz
		ld	a,%100
		ld	[JumpFlag],a
		call	CheckJumpCol
		ld	a,[JumpFlag]
		bit	4,a
		jp	z,NotOnFloorJ
		jr	SetJumpCount
		
SetNewJump	xor	a
		ld	[JumpFlag],a
		call	CheckJumpCol
		ld	a,[JumpFlag]
		bit	4,a
		jp	z,NotOnFloorJ
		
SetJumpCount	ld	a,1
		ld	[JumpCount],a

		ld	a,[JumpFlag]
		ld	hl,CharFlag
		bit	0,[hl]
		jp	nz,SetJumpRight
		bit	1,[hl]
		jp	nz,SetJumpLeft

CheckVertMoveH	ld	a,[JumpFlag]
		bit	1,a
		jp	nz,CheckHorizMove
		bit	2,a
		jp	nz,CheckHorizMove
		
		jp	CheckHorizMove

SetJumpLeft	or	%10000001
		ld	[JumpFlag],a
		ld	hl,CharFrame
		set	2,[hl]
		jp	CheckVertMoveH

SetJumpRight	
		or	%10000000
		ld	[JumpFlag],a
		ld	hl,CharFrame
		res	2,[hl]
		jp	CheckVertMoveH

NotOnFloorJ	jp	CheckVertMove
		
CheckJumpCol	ld	a,[ColFlag]
		bit	0,a
		jr	nz,JumpEn
		bit	1,a
		jr	nz,JumpEnL
		bit	2,a
		jr	nz,JumpEnL
		ret
JumpEn		ld	a,[JumpFlag]
		set	4,a
		ld	[JumpFlag],a
		ret		

JumpEnL		ld	a,[CharFlag]
		bit	0,a
		jr	nz,SetJumpOffLad
		bit	1,a
		jr	nz,SetJumpOffLad
		ret
SetJumpOffLad	ld	hl,JumpFlag
		set	1,[hl]
		set	4,[hl]
		ld	hl,CharFlag
		res	2,[hl]
		res	3,[hl]
		ret		

;************************************************************************

DoCollisionsR	ld	hl,ColFlagR
		ld	a,[hl]
		ld	[ColFlagROld],a
		xor	a
		ld	[hl],a

CheckColR0a	ld	b,3		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapRobCollision
		cp	wall
		jp	z,SetColR0a
		cp	floor0
		jp	z,SetColR0a
		cp	floor1
		jp	z,SetColR0a
		cp	floor2
		jp	z,SetColR0a
		jp	CheckColR0b
SetColR0a		
		set	0,[hl]

CheckColR0b	ld	b,6
		ld	c,16
		call	MapRobCollision
		cp	wall
		jp	z,SetColR0b
		cp	floor0
		jr	z,SetColR0b
		cp	floor1
		jr	z,SetColR0b
		cp	floor2
		jr	z,SetColR0b
		jr	CheckColR0c
SetColR0b		
		set	0,[hl]

CheckColR0c	ld	b,9
		ld	c,16
		call	MapRobCollision
		cp	wall
		jp	z,SetColR0c
		cp	floor0
		jr	z,SetColR0c
		cp	floor1
		jr	z,SetColR0c
		cp	floor2
		jr	z,SetColR0c
		jr	CheckColR0d
SetColR0c		
		set	0,[hl]

CheckColR0d	ld	b,12
		ld	c,16
		call	MapRobCollision
		cp	wall
		jp	z,SetColR0d
		cp	floor0
		jr	z,SetColR0d
		cp	floor1
		jr	z,SetColR0d
		cp	floor2
		jr	z,SetColR0d
		jr	CheckColR1
SetColR0d		
		set	0,[hl]

CheckColR1	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,7
		call	MapRobCollision
		cp	ladl
		jp	nz,CheckColR2
		ld	b,15
		ld	c,7
		call	MapRobCollision
		cp	ladr
		jp	nz,CheckColR2
		set	1,[hl]

CheckColR2	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,16
		call	MapRobCollision
		cp	ladl
		jp	nz,CheckColR3
		ld	b,15
		ld	c,16
		call	MapRobCollision
		cp	ladr
		jp	nz,CheckColR3
		set	2,[hl]

CheckColR3	ld	b,255		; Check 2 - check for wall hit to left of player
		ld	c,15
		call	MapRobCollision
		cp	wall
		jr	z,SetColR3
		cp	floor0
		jr	z,SetColR3
		cp	floor1
		jr	z,SetColR3
		cp	floor2
		jr	z,SetColR3
		jr	CheckColR4
SetColR3		
		set	3,[hl]

CheckColR4	ld	b,16		; Check 3 - check for wall hit to right of player
		ld	c,15
		call	MapRobCollision
		cp	wall
		jr	z,SetColR4
		cp	floor0
		jr	z,SetColR4
		cp	floor1
		jr	z,SetColR4
		cp	floor2
		jr	z,SetColR4
		jr	CheckColR5a
SetColR4		
		set	4,[hl]

CheckColR5a	ld	b,255		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapRobCollision
		cp	wall
		jr	z,SetColR5a
		cp	floor0
		jr	z,SetColR5a
		cp	floor1
		jr	z,SetColR5a
		cp	floor2
		jr	z,SetColR5a
		jr	CheckColR6a
SetColR5a	ld	b,255
		ld	c,15
		call	MapRobCollision
		cp	wall
		jr	z,CheckColR6a
		cp	floor0
		jr	z,CheckColR6a
		cp	floor1
		jr	z,CheckColR6a
		cp	floor2
		jr	z,CheckColR6a
		set	5,[hl]

CheckColR6a	ld	b,16		; Check 0abcd - Check for platform under player
		ld	c,16
		call	MapRobCollision
		cp	wall
		jr	z,SetColR6a
		cp	floor0
		jr	z,SetColR6a
		cp	floor1
		jr	z,SetColR6a
		cp	floor2
		jr	z,SetColR6a
		jr	EndCheckColR
SetColR6a	ld	b,16
		ld	c,15
		call	MapRobCollision
		cp	wall
		jr	z,CheckColR7
		cp	floor0
		jr	z,CheckColR7
		cp	floor1
		jr	z,CheckColR7
		cp	floor2
		jr	z,CheckColR7
		set	6,[hl]

CheckColR7	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,16
		call	MapRobCollision
		cp	floor3
		jp	nz,EndCheckColR
		ld	b,15
		ld	c,16
		call	MapRobCollision
		cp	floor3
		jp	nz,EndCheckColR
		set	7,[hl]

EndCheckColR	ret
		
; ColFlagR:	Collisions:
;
; bit 0 - Platform (3-12,16)
; bit 1 - Ladder (0-15,7)
; bit 2 - Ladder (0-15,16)
; bit 3 - Wall Left (0,15)
; bit 4 - Wall Right (15,15)
; bit 5 - Lad-Plat L
; bit 6 - Lad-Plat R
; bit 7 - Plat hole (floor3) detected underfoot (0-15,16)
;
;
; RobFlag:	Actions:
;
; bit 0 - Direction 0=R/U, 1=L/D
; bit 1 - Direction 0=H, 1=V
; bit 2 - 0=Robot/Char on Same Level, 1=Above/Below
; bit 3 - 0=Robot Above char, 1=Robot Below char
; bit 4 - 0=Off ladder, 1=Just off ladder (last frame)
; bit 5 - 1=Trapped, just bounced off wall when going towards char - on same level (search for a ladder)
; bit 6 - 1=Falling down hole
; bit 7 -

;************************************************************************

MoveRobot	;call	SetBackBlack
		ld	a,[RobFlag]
		bit	6,a
		jr	nz,ProcessRobFall

		ld	a,[ColFlagR]
		bit	7,a
		jp	z,CheckRobTrapped

RobFallDownHole	ld	hl,RobFlag
		set	6,[hl]
		xor	a
		ld	[RobFallenCount],a
		ld	[RobFallenFlag],a

ProcessRobFall	ld	hl,RobFlag
		res	7,[hl]

		ld	a,[RobFallenCount]
		inc	a
		ld	[RobFallenCount],a

DoFallAction	ld	a,[RobFallenFlag]
		cp	0		;  Fall action 0, 16 steps of fall into hole
		jr	z,FallAction0
		cp	1		;  Robot makes floor3 into floor2
		jr	z,FallAction1
		cp	2		;  Robot makes floor2 into floor1
		jr	z,FallAction2
		cp	3		;  Robot makes floor1 into floor0,
		jr	z,FallAction3
		cp	4		;  Robot raises 16 steps slowly, End of actions
		jr	z,FallAction4
		cp	5		;  Robot caught, now falling down to next plat and break up
		jr	z,FallAction5

FallAction0	;call	SetBackRed
		ld	a,[RobFallenCount]
		cp	17
		jp	z,SetNextFallAction
		
		ld	a,[RobY]
		inc	a
		ld	[RobY],a
		ret

FallAction1	;call	SetBackGreen
		ld	hl,RobFlag
		set	7,[hl]
		ld	a,[RobFallenCount]
		cp	80
		jp	z,SetNextFallAction
		cp	0
		jp	nz,CheckFallHole
		ld	a,2
		jp	SetRobotFallBlock
		
FallAction2	;call	SetBackBlue
		ld	hl,RobFlag
		set	7,[hl]
		ld	a,[RobFallenCount]
		cp	80
		jp	z,SetNextFallAction
		jp	CheckFallHole
		cp	0
		ret	;nz
		ld	a,2
		jp	SetRobotFallBlock
		
FallAction3	;call	SetBackPurple
		ld	hl,RobFlag
		set	7,[hl]
		ld	a,[RobFallenCount]
		cp	80
		jr	z,SetNextFallAction
		jr	CheckFallHole
		cp	0
		ret	;nz
		ld	a,2
		jp	SetRobotFallBlock
		
FallAction4	;call	SetBackYellow
		ld	a,[RobFallenCount]
		cp	64
		jr	z,EndFallRobot
		ld	bc,-64
		ld	a,[RobY]
		ld	h,a
		ld	a,[RobYl]
		ld	l,a
		add	hl,bc
		ld	a,h
		ld	[RobY],a
		ld	a,l
		ld	[RobYl],a	
		ret		
		
FallAction5	ld	a,[ColFlagR]
		and	%10000001
		cp	%00000001
		jr	z,RobotHitPlatDead
		cp	%10000000
		jp	z,RobFallDownHole

		ld	a,[RobY]
		inc	a
		ld	[RobY],a
		ret	

RobotHitPlatDead	;call	SetBackPurple
		ret	
		
SetNextFallAction	xor	a
		ld	[RobFallenCount],a
		ld	a,[RobFallenFlag]
		inc	a
		ld	[RobFallenFlag],a
		jp	DoFallAction

SetRobotFallBlock	ld	b,4
		ld	c,4
		call	PlaceBlockScrR
		ld	b,12
		ld	c,4
		call	PlaceBlockScrR
		ret

CheckFallHole	ld	b,0		; Check 1 - check for ladder alignment to player
		ld	c,3
		call	MapRobCollision
		cp	floor3
		jp	z,FallHoleFound
		ld	b,15
		ld	c,3
		call	MapRobCollision
		cp	floor3
		jp	z,FallHoleFound
		ret
FallHoleFound	xor	a
		ld	[RobFallenCount],a
		ld	a,5
		ld	[RobFallenFlag],a
		ld	a,[RobY]
		inc	a
		inc	a
		inc	a
		ld	[RobY],a
		ret


EndFallRobot	ld	hl,RobFlag
		res	6,[hl]
		ld	hl,ColFlagR
		res	7,[hl]
		
CheckRobTrapped	call	CheckRobCharLvl
		ld	a,[RobFlag]
		bit	5,a
		jr	z,CheckRobVert

RobotFindLadder	ld	a,[ColFlagR]
		and	%110
		cp	%010
		jp	z,SetRobotUpward	
		cp	%100
		jp	z,SetRobotDownwd	
		cp	%110
		jp	z,SetRobotDownwd	

CheckRobVert	ld	a,[RobFlag]
		bit	1,a
		jr	nz,MoveRobotVert

MoveRobotHoriz	bit	0,a
		jp	nz,MoveRobotLeft
		jp	MoveRobotRight




MoveRobotVert	bit	0,a
		jp	nz,MoveRobotDown

MoveRobotUp	ld	a,[ColFlagR]
		and	%01100111
		cp	%00100110
		jp	z,RobotHitPlatUChk
		cp	%01000110
		jp	z,RobotHitPlatUChk
		
		ld	a,[ColFlagR]
		and	%111
		cp	%100
		jp	z,RobotHitPlatU
		
ContMoveRobU	ld	a,[RobSpeedNeg]
		ld	b,a
		ld	a,[RobSpeedNegl]
		ld	c,a
		
		ld	a,[RobY]
		ld	h,a
		ld	a,[RobYl]
		ld	l,a
		add	hl,bc
		ld	a,h
		ld	[RobY],a
		ld	a,l
		ld	[RobYl],a
		ret

RobotHitPlatUChk	ld	hl,RobFlag
		set	4,[hl]

		ld	a,[RobFlag]
		bit	2,a
		jr	z,RobotHitPlatU
		bit	3,a
		jr	z,RobotHitPlatU
		jp	ContMoveRobU
		
RobotHitPlatU	ld	hl,RobFlag
		set	4,[hl]
		res	1,[hl]

		ld	a,[CharX]		; CharX + 8
		ld	b,8
		ld	c,8
		add	b
		ld	b,a
		ld	a,[RobX]		; RobX + 8
		add	c
		sub	b
		ld	[DiffY],a
		jp	c,RobLeftOfChar
		jp	z,RobMidOfChar
		jp	RobRightOfChar


MoveRobotDown	ld	a,[ColFlagR]
		and	%01100111
		cp	%00100110
		jr	z,RobotHitPlatDChk
		cp	%01000110
		jr	z,RobotHitPlatDChk
		
		ld	a,[ColFlagR]
		and	%001
		cp	%001
		jr	z,RobotHitPlatD
		
ContMoveRobD	ld	a,[RobSpeedPos]
		ld	b,a
		ld	a,[RobSpeedPosl]
		ld	c,a
		
		ld	a,[RobY]
		ld	h,a
		ld	a,[RobYl]
		ld	l,a
		add	hl,bc
		ld	a,h
		ld	[RobY],a
		ld	a,l
		ld	[RobYl],a
		ret

RobotHitPlatDChk	ld	hl,RobFlag
		set	4,[hl]
		
		ld	a,[RobFlag]
		bit	2,a
		jr	z,RobotHitPlatD
		bit	3,a
		jr	nz,RobotHitPlatD
		jp	ContMoveRobD
		
RobotHitPlatD	ld	hl,RobFlag
		set	4,[hl]
		res	1,[hl]
		
		ld	a,[CharX]		; CharX + 8
		ld	b,8
		ld	c,8
		add	b
		ld	b,a
		ld	a,[RobX]		; RobX + 8
		add	c
		sub	b
		ld	[DiffY],a
		jr	c,RobLeftOfChar
		jr	z,RobMidOfChar
		jr	RobRightOfChar

RobLeftOfChar	ld	hl,RobFlag
		res	0,[hl]
		res	1,[hl]
		ret

RobMidOfChar	ld	hl,RobFlag
		ret

RobRightOfChar	ld	hl,RobFlag
		set	0,[hl]
		res	1,[hl]
		ret

MoveRobotRight	ld	a,[RobFlag]
		bit	4,a
		jr	nz,RobotSameLvlRx
		bit	2,a
		jr	z,RobotSameLvlR
		bit	3,a
		jr	z,RobotAboveCharR
		jr	RobotBelowCharR

RobotBelowCharR	ld	a,[ColFlagR]
		and	%110
		cp	%110
		jp	z,SetRobotUpward
		cp	%010
		jp	z,SetRobotUpward
		jr	RobotSameLvlR

RobotAboveCharR	ld	a,[ColFlagR]
		and	%110
		cp	%110
		jp	z,SetRobotDownwd
		cp	%100
		jp	z,SetRobotDownwd
		jr	RobotSameLvlR

RobotSameLvlRx	ld	a,[ColFlagR]
		and	%00000111
		cp	%00000001
		jr	nz,RobotSameLvlR
		res	4,[hl]

RobotSameLvlR	ld	hl,RobFlag
		ld	a,[ColFlagR]
		bit	4,a
		jr	nz,RobotHitWallR

		ld	a,[RobSpeedPos]
		ld	b,a
		ld	a,[RobSpeedPosl]
		ld	c,a
		
		ld	a,[RobX]
		ld	h,a
		ld	a,[RobXl]
		ld	l,a
		add	hl,bc
		ld	a,h
		ld	[RobX],a
		ld	a,l
		ld	[RobXl],a
		ret

RobotHitWallR	ld	a,[RobFlag]
		bit	2,a
		jr	nz,RobotHitWallRC

		ld	a,[CharX]		; CharX + 8
		ld	b,8
		ld	c,8
		add	b
		ld	b,a
		ld	a,[RobX]		; RobX + 8
		add	c
		sub	b
		ld	[DiffY],a
		jr	c,RobotPathBlockR
		
RobotHitWallRC	ld	hl,RobFlag
		set	0,[hl]
		res	5,[hl]
		ret
		
RobotPathBlockR	ld	hl,RobFlag
		set	0,[hl]
		set	5,[hl]
		ret




MoveRobotLeft	ld	a,[RobFlag]
		bit	4,a
		jr	nz,RobotSameLvlLx
		bit	2,a
		jr	z,RobotSameLvlL
		bit	3,a
		jr	z,RobotAboveCharL
		jr	RobotBelowCharL

RobotBelowCharL	ld	a,[ColFlagR]
		and	%110
		cp	%110
		jp	z,SetRobotUpward
		cp	%010
		jp	z,SetRobotUpward
		jr	RobotSameLvlL

RobotAboveCharL	ld	a,[ColFlagR]
		and	%110
		cp	%110
		jp	z,SetRobotDownwd
		cp	%100
		jp	z,SetRobotDownwd
		jr	RobotSameLvlL

RobotSameLvlLx	ld	a,[ColFlagR]
		and	%00000111
		cp	%00000001
		jr	nz,RobotSameLvlL
		res	4,[hl]

RobotSameLvlL	ld	hl,RobFlag
		;res	4,[hl]
		ld	a,[ColFlagR]
		bit	3,a
		jr	nz,RobotHitWallL
		
		ld	a,[RobSpeedNeg]
		ld	b,a
		ld	a,[RobSpeedNegl]
		ld	c,a
		
		ld	a,[RobX]
		ld	h,a
		ld	a,[RobXl]
		ld	l,a
		add	hl,bc
		ld	a,h
		ld	[RobX],a
		ld	a,l
		ld	[RobXl],a
		ret

RobotHitWallL	ld	a,[RobFlag]
		bit	2,a
		jr	nz,RobotHitWallLC

		ld	a,[CharX]		; CharX + 8
		ld	b,8
		ld	c,8
		add	b
		ld	b,a
		ld	a,[RobX]		; RobX + 8
		add	c
		sub	b
		ld	[DiffY],a
		jr	c,RobotHitWallLC
		jr	z,RobotHitWallLC
		jr	RobotPathBlockL
		
RobotHitWallLC	ld	hl,RobFlag
		res	0,[hl]
		res	5,[hl]
		ret
		
RobotPathBlockL	ld	hl,RobFlag
		res	0,[hl]
		set	5,[hl]
		ret

CheckRobCharLvl	ld	a,[CharYNJ]
		ld	b,a
		ld	a,[RobY]		; RobY
		sub	b
		ld	[DiffY],a
		jr	c,RobAboveChar
		jr	z,RobSameLevel
		jr	RobBelowChar
		
RobAboveChar	ld	hl,RobFlag
		set	2,[hl]
		res	3,[hl]
		ret

RobSameLevel	ld	hl,RobFlag
		res	2,[hl]
		res	3,[hl]
		ret

RobBelowChar	ld	hl,RobFlag
		set	2,[hl]
		set	3,[hl]
		ret

SetRobotUpward	ld	hl,RobFlag
		res	0,[hl]
		set	1,[hl]
		res	5,[hl]
		ret

SetRobotDownwd	ld	hl,RobFlag
		set	0,[hl]
		set	1,[hl]
		res	5,[hl]
		ret






; ColX & ColY = collision co-ords for map check, A reg = char byte at map position

;************************************************************************
MapCharCollision	push	hl
		ld	a,[CharX]
		add	b
		rr	a
		rr	a
		rr	a
		and	31
		ld	d,0
		ld	e,a
		ld	a,[CharY]
		add	c
		rr	a
		rr	a
		and	%00111110
		ld	b,0
		ld	c,a
		ld	hl,MapJumpTab
		add	hl,bc
		ld	a,[hl+]
		ld	c,a
		ld	a,[hl]
		ld	b,a
		ld	hl,$c200
		add	hl,bc
		add	hl,de
		ld	a,[hl]
		pop	hl
		ret

;************************************************************************
MapRobCollision	push	hl
		ld	a,[RobX]
		add	b
		rr	a
		rr	a
		rr	a
		and	31
		ld	d,0
		ld	e,a
		ld	a,[RobY]
		add	c
		rr	a
		rr	a
		and	%00111110
		ld	b,0
		ld	c,a
		ld	hl,MapJumpTab
		add	hl,bc
		ld	a,[hl+]
		ld	c,a
		ld	a,[hl]
		ld	b,a
		ld	hl,$c200
		add	hl,bc
		add	hl,de
		ld	a,[hl]
		pop	hl
		ret

;************************************************************************

PlaceBlockScreen	push	hl
		push	af
		ld	a,[CharX]
		add	b
		rr	a
		rr	a
		rr	a
		and	31
		ld	d,0
		ld	e,a
		ld	a,[CharY]
		add	c
		rr	a
		rr	a
		and	%00111110
		ld	b,0
		ld	c,a
		ld	hl,MapJumpTab
		add	hl,bc
		ld	a,[hl+]
		ld	c,a
		ld	a,[hl]
		ld	b,a

		ld	hl,$9800
		add	hl,bc
		add	hl,de
		
WaitVRamLoop0	ldh	a,[rSTAT]          	; Wait until you can write to vram
		and	2              
		jr	nz,WaitVRamLoop0

		pop	af
		ld	[hl],a

		ld	hl,$c200
		add	hl,bc
		add	hl,de
		ld	[hl],a
		pop	hl
		ret

;************************************************************************
PlaceBlockScrR	push	hl
		push	af
		ld	a,[RobX]
		add	b
		rr	a
		rr	a
		rr	a
		and	31
		ld	d,0
		ld	e,a
		ld	a,[RobY]
		add	c
		rr	a
		rr	a
		and	%00111110
		ld	b,0
		ld	c,a
		ld	hl,MapJumpTab
		add	hl,bc
		ld	a,[hl+]
		ld	c,a
		ld	a,[hl]
		ld	b,a

		ld	hl,$9800
		add	hl,bc
		add	hl,de
		
WaitVRamLoop1	ldh	a,[rSTAT]          	; Wait until you can write to vram
		and	2              
		jr	nz,WaitVRamLoop1

		pop	af
		ld	[hl],a

		ld	hl,$c200
		add	hl,bc
		add	hl,de
		ld	[hl],a
		pop	hl
		ret

;************************************************************************

ReadPad		ld	a,$10
		ldh	[$0],a
		ldh	a,[$0]
		ldh	a,[$0]
		cpl
		and	a,$0F
		swap	a
		ld	b,a

		ld	a,$20
		ldh	[$0],a
		ldh	a,[$0]
		ldh	a,[$0]
		ldh	a,[$0]
		ldh	a,[$0]
		ldh	a,[$0]
		ldh	a,[$0]
		cpl
		and	a,$0F
		or	a,b
		ld	b,a

		ld	a,[PadData]
		xor	a,b
		and	a,b
		ld	[PadDataEdge],a
		ld	a,b
		ld	[PadData],a	

		ld	a,$30
		ldh	[$0],a
		ret

; bit 0 - right, 1 - left, 2 - up, 3 - down, 4 - A, 5 - B, 6 - sel, 7 - start

;************************************************************************

ShowHL		ld	a,h
		and	a,$f
		add	32
		ld	[$9800+1],a
		ld	a,h
		swap	a
		and	a,$f
		add	32
		ld	[$9800+0],a
		ld	a,l
		and	a,$f
		add	32
		ld	[$9800+3],a
		ld	a,l
		swap	a
		and	a,$f
		add	32
		ld	[$9800+2],a
		ret

;************************************************************************

ShowVal		Push	af
		and	a,$f
		add	32
		inc	hl
		ld	[hl],a
		Pop	af
		swap	a
		and	a,$f
		add	32
		dec	hl
		ld	[hl],a
		inc	hl
		inc	hl
		ret

;************************************************************************

; a=val, hl=screen addr
ShowBinary	ld	b,8
		ld	c,a
ShowBitLoop	rlc	c
		jp	c,ShowBit1
ShowBit0		ld	a,40
		ld	[hl+],a
		jp	ShowBitCont
ShowBit1		ld	a,41
		ld	[hl+],a
ShowBitCont	dec	b
		jp	nz,ShowBitLoop
		ret

;************************************************************************

SetBackRed	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%00011111
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%00000000
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************

SetBackGreen	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%11100000
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%00000011
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************
SetBackBlue	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%00000000
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%01111100
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************
SetBackPurple	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%00011111
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%01111100
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************
SetBackWhite	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%11111111
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%01111111
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************
SetBackYellow	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%11111111
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%00000011
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************

SetBackBlack	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%00000000
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%00000000
		ldh	[$69],a		; set colour data HIGH
		ret


;************************************************************************

SetBackCyan	ld	a,%00000110
		ldh	[$68],a		; set palette specification reg LOW
		ld	a,%11100000
		ldh	[$69],a		; set colour data LOW		
		ld	a,%00000111
		ldh	[$68],a		; set palette specification reg HIGH
		ld	a,%01111111
		ldh	[$69],a		; set colour data HIGH
		ret

;************************************************************************

ShowDiagnostics	ld	a,[DiagFlag]
		bit	0,a
		ret	z
		ld	a,[JumpFlag]
		ld	hl,$9844
		call	ShowBinary
		ld	a,[JumpCount]
		ld	hl,$9864
		call	ShowBinary
		
		ld	a,[ColFlag]
		ld	hl,$984d
		call	ShowBinary
		ld	a,[ColFlagR]
		ld	hl,$986d
		call	ShowBinary
		ld	a,[CharYNJ]
		ld	hl,$98ad
		call	ShowBinary
		ret














